@@ -8,12 +8,12 @@ from numba import jit
 from pandas.tseries.offsets import BDay
 from pathlib import Path
 from multiprocessing import shared_memory
-import subprocess
+
 from subprocess import run, PIPE
-import boto3
+from datetime import datetime, timedelta
 
 from SharedData.Logger import Logger
-from SharedData.SharedDataAWS import S3SyncDownloadTimeSeries
+from SharedData.SharedDataAWSS3 import S3SyncDownloadTimeSeries,S3Upload
 
 class SharedDataTimeSeries:
 
@@ -62,7 +62,20 @@ class SharedDataTimeSeries:
             if isCreate:
                 if self.sharedData.s3read:
                     path, shm_name = self.getDataPath()
-                    S3SyncDownloadTimeSeries(path, shm_name)
+                    path = str(path)
+                    # Sync download S3 files
+                    lastsync = self.sharedData.lastsync
+                    if not path in lastsync.index:
+                        S3SyncDownloadTimeSeries(path, shm_name)
+                        lastsync.loc[path,'timestamp'] = datetime.utcnow()
+                        lastsync.index.name = 'file'
+                        lastsync.to_csv(self.sharedData.lastsyncfpath)
+                    else:
+                        td = (datetime.utcnow() - lastsync['timestamp'][path]).seconds/86400
+                        if td>self.sharedData.sync_frequency_days:
+                            S3SyncDownloadTimeSeries(path, shm_name)
+                            lastsync.loc[path,'timestamp'] = datetime.utcnow()
+                            lastsync.to_csv(self.sharedData.lastsyncfpath)
                 self.Read()                
         
         else: # map existing dataframe
@@ -171,13 +184,13 @@ class SharedDataTimeSeries:
     # C R U D
     def Malloc(self, value=None):
         tini=time.time()
-        Logger.log.debug('Malloc %s/%s/%s ...%.2f%% ' % \
-            (self.feeder,self.period,self.tag,0.0))        
+        
 
         #Create write ndarray
         path, shm_name = self.getDataPath()
         fpath = path / ('shm_info.json')
 
+        Logger.log.debug('Malloc %s ...%.2f%% ' % (shm_name,0.0))
         try: # try create memory file
             rows = len(self.index)
             cols = len(self.columns)
@@ -210,8 +223,8 @@ class SharedDataTimeSeries:
                     }
                 json.dump(shm_info, outfile, indent=3)
 
-            Logger.log.debug('Malloc create %s/%s/%s ...%.2f%% %.2f sec! ' % \
-                (self.feeder,self.period,self.tag,100,time.time()-tini))            
+            Logger.log.debug('Malloc create %s ...%.2f%% %.2f sec! ' % \
+                (shm_name,100,time.time()-tini))            
 
             return True
         except:
@@ -255,8 +268,7 @@ class SharedDataTimeSeries:
         nfiles = len(files.index)
         if nfiles>0:
             tini=time.time() 
-            Logger.log.debug('Reading %s/%s/%s ...%.2f%% ' % \
-                (self.feeder,self.period,self.tag,0.0))   
+            Logger.log.debug('Reading %s ...%.2f%% ' % (shm_name,0.0))   
             n=0
             for f in files.index:
                 fpath=files.loc[f,'fpath']            
@@ -275,8 +287,8 @@ class SharedDataTimeSeries:
                     self.setValuesJit(self.data.values,tidx,sidx,arr[:,1:])
                 n=n+1                
 
-            Logger.log.debug('Reading %s/%s/%s ...%.2f%% %.2f sec! ' % \
-                (self.feeder,self.period,self.tag,100*(n/nfiles),time.time()-tini))
+            Logger.log.debug('Reading %s ...%.2f%% %.2f sec! ' % \
+                (shm_name,100*(n/nfiles),time.time()-tini))
 
     def Write(self, busdays=None, startDate=None):
         tini = time.time()
@@ -290,13 +302,14 @@ class SharedDataTimeSeries:
         if not startDate is None:
             lastdate = startDate
         
-        startdatestr = lastdate.strftime('%Y-%m-%d')
-        Logger.log.debug('Writing %s/%s/%s from %s ...%.2f%% ' % \
-            (self.feeder,self.period,self.tag,startdatestr,0.0))
-        
         path, shm_name = self.getDataPath()        
         if not os.path.isdir(path):
             os.makedirs(path)        
+
+        startdatestr = lastdate.strftime('%Y-%m-%d')
+        Logger.log.debug('Writing %s from %s ...%.2f%% ' % \
+            (shm_name,startdatestr,0.0))
+        
             
         years = self.data.loc[lastdate:,:].index.year.unique()
         ny = len(years)
@@ -316,10 +329,10 @@ class SharedDataTimeSeries:
                 with open(fpath_csv, 'w') as f:
                     f.write(cols)
                 if self.sharedData.s3write:
-                    self.S3SyncUpload(fpath_npy)
-                    self.S3SyncUpload(fpath_csv)
+                    S3Upload(fpath_npy)
+                    S3Upload(fpath_csv)
                 cy=cy+1      
 
-        Logger.log.debug('Writing %s/%s/%s from %s ...%.2f%% %.2f sec!' % \
-            (self.feeder,self.period,self.tag,startdatestr,100*cy/ny,time.time()-tini))        
+        Logger.log.debug('Writing %s from %s ...%.2f%% %.2f sec!' % \
+            (shm_name,startdatestr,100*cy/ny,time.time()-tini))        
         
\ No newline at end of file
